/*
 * Copyright (C) 2002-2005 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.sqlPlugin.browser;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.wm.ToolWindow;
import com.intellij.openapi.wm.ToolWindowManager;

import com.kiwisoft.db.Database;
import com.kiwisoft.db.DatabaseColumn;
import com.kiwisoft.db.DatabaseManager;
import com.kiwisoft.db.DatabaseTable;
import com.kiwisoft.db.driver.MySQLDriver;
import com.kiwisoft.sqlPlugin.IconNames;
import com.kiwisoft.sqlPlugin.QueryPanel;
import com.kiwisoft.sqlPlugin.SQLPluginPanel;
import com.kiwisoft.sqlPlugin.SQLQueryPlugin;
import com.kiwisoft.sqlPlugin.actions.AboutAction;
import com.kiwisoft.sqlPlugin.actions.HelpAction;
import com.kiwisoft.sqlPlugin.actions.PropertiesAction;
import com.kiwisoft.utils.*;
import com.kiwisoft.utils.gui.IconManager;
import com.kiwisoft.utils.gui.TristateBoxIcon;
import com.kiwisoft.utils.gui.inspect.InfoNode;
import com.kiwisoft.utils.gui.table.SortableTable;
import com.kiwisoft.utils.gui.table.PropertiesTableModel;
import com.kiwisoft.utils.gui.tree.DynamicTree;
import com.kiwisoft.utils.gui.tree.DynamicTreeNode;
import com.kiwisoft.utils.gui.tree.DynamicTreeNodeRenderer;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.7 $, $Date: 2005/10/29 11:18:39 $
 */
public class BrowserPanel extends JPanel
{
	private static Map instanceMap=new WeakHashMap();

	public static BrowserPanel getInstance(Project project)
	{
		BrowserPanel instance=(BrowserPanel)instanceMap.get(project);
		if (instance==null)
		{
			try
			{
				instance=new BrowserPanel(project);
				instanceMap.put(project, instance);
			}
			catch (Exception e)
			{
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			}
		}
		return instance;
	}

	public static void closeInstance(Project project)
	{
		BrowserPanel browserPanel=(BrowserPanel)instanceMap.get(project);
		if (browserPanel!=null) browserPanel.dispose();
		instanceMap.remove(project);
	}

	private JScrollPane treePane;
	private Map treeMap=new HashMap();
	private Project project;
	private SelectionListener selectionListener;
	private MouseListener mouseListener;
	private PropertiesTableModel tmProperties;
	private JComboBox cbxDatabases;
	private JComboBox cbxGroups;
	private ListenerSupport listenerSupport=new ListenerSupport();

	public static void main(String[] args)
	{
		Locale.setDefault(Locale.ENGLISH);
		Utils.setDialogWrapper(false);

		UIManager.put("TristateBoxUI", "com.kiwisoft.utils.gui.TristateBoxUI");
		UIManager.put("TristateBox.icon", new TristateBoxIcon());
		UIManager.put("TristateBox.focus", UIManager.getColor("CheckBox.focus"));
		UIManager.put("TristateBox.disabledText", UIManager.getColor("CheckBox.disabledText"));
		UIManager.put("TristateBox.focusInputMap", new UIDefaults.LazyInputMap(new Object[]{
			"SPACE", "pressed",
			"released SPACE", "released"
		}));

		DatabaseManager databaseManager=DatabaseManager.getInstance(null);

		Database database=new Database("Test");
		database.setDriver("jdbc:mysql");
		database.setProperty(MySQLDriver.HOST, "localhost");
		database.setProperty(MySQLDriver.PORT, new Integer(3306));
		database.setProperty(MySQLDriver.DATABASE, "test");
		database.setProperty(MySQLDriver.USER, "mysql");
		database.setProperty(MySQLDriver.AUTO_COMMIT, Boolean.FALSE);
		databaseManager.addDatabase(database);

		databaseManager.setCurrentDatabase(database);

		JFrame frame=new JFrame("SQL Schema");
		frame.getContentPane().add(new BrowserPanel(null));
		frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		frame.setIconImage(IconManager.getIcon(SQLPluginPanel.class.getClassLoader(), IconNames.DATABASE_STRUCTURE).getImage());
		frame.setSize(800, 300);
		frame.setVisible(true);
	}

	public BrowserPanel(Project project)
	{
		this.project=project;
		initializeComponents();
	}

	private void initializeComponents()
	{
		DynamicTree tree=getTree(null);
		treePane=new JScrollPane(tree);

		tmProperties=new PropertiesTableModel(project, null);
		JTable tblProperties=new SortableTable(tmProperties);

		JSplitPane splitPane=new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		splitPane.setDividerLocation(500);
		splitPane.setOneTouchExpandable(true);
		splitPane.add(treePane);
		splitPane.add(new JScrollPane(tblProperties));
		splitPane.setPreferredSize(new Dimension(500, 300));

		DatabaseManager databaseManager=DatabaseManager.getInstance(project);
		CurrentDatabaseListener currentDatabaseListener=new CurrentDatabaseListener();

		List groups=new LinkedList(databaseManager.getGroups());
		Collections.sort(groups);
		groups.add(0, DatabaseManager.ALL_GROUPS);
		cbxGroups=new JComboBox(groups.toArray());
		cbxGroups.setActionCommand("group");
		listenerSupport.installActionListener(cbxGroups, currentDatabaseListener);

		String currentGroup=databaseManager.getCurrentGroup();
		Database currentDatabase=databaseManager.getCurrentDatabase();
		List databases=new LinkedList(databaseManager.getDatabases(currentGroup));
		databases.add(DatabaseManager.NO_DATABASE);

		cbxDatabases=new JComboBox(databases.toArray());
		cbxDatabases.setActionCommand("database");
		cbxDatabases.addActionListener(currentDatabaseListener);
		listenerSupport.installActionListener(cbxDatabases, currentDatabaseListener);

		if (currentGroup!=null)
			cbxGroups.setSelectedItem(currentGroup);
		else
			cbxGroups.setSelectedItem(DatabaseManager.ALL_GROUPS);

		if (currentDatabase!=null)
		{
			cbxDatabases.setSelectedItem(currentDatabase);
			cbxDatabases.setToolTipText(currentDatabase.getURL());
		}
		else
			cbxDatabases.setSelectedItem(DatabaseManager.NO_DATABASE);

		JToolBar toolbar=new JToolBar();
		toolbar.setFloatable(false);
		toolbar.add(cbxDatabases);
		toolbar.add(cbxGroups);
		toolbar.add(Utils.createButton(new PropertiesAction(project)));
		toolbar.add(Utils.createButton(new RefreshAction()));
		toolbar.addSeparator();
		toolbar.add(Utils.createButton(new CancelAction()));
		toolbar.addSeparator();
		toolbar.add(Utils.createButton(new HelpAction()));
		toolbar.add(Utils.createButton(new AboutAction(this)));

		setLayout(new BorderLayout());
		add(toolbar, BorderLayout.NORTH);
		add(splitPane, BorderLayout.CENTER);

		listenerSupport.installObserver(databaseManager, new DatabaseManagerObserver());

		selectionListener=new SelectionListener();
		mouseListener=new TreeMouseListener();

		setTree(getTree(currentDatabase));
	}

	public void dispose()
	{
		listenerSupport.dispose();
	}

	public void removeNotify()
	{
		DatabaseManager.getInstance(project).closeAllConnections();
		super.removeNotify();
	}

	private void showProperties(PropertyHolder propertyHolder)
	{
		tmProperties.setPropertyHolder(propertyHolder);
	}

	private void setTree(DynamicTree tree)
	{
		treePane.setViewportView(tree);
		tree.addTreeSelectionListener(selectionListener);
		tree.addMouseListener(mouseListener);
		updateProperties(tree.getSelectionPath());
	}

	private void updateProperties(TreePath selectionPath)
	{
		if (selectionPath!=null)
		{
			DynamicTreeNode treeNode=(DynamicTreeNode)selectionPath.getLastPathComponent();
			Object userObject=treeNode.getUserObject();
			if (userObject instanceof PropertyHolder)
			{
				showProperties((PropertyHolder)userObject);
				return;
			}
		}
		showProperties(null);
	}

	private class SelectionListener implements TreeSelectionListener
	{
		public void valueChanged(TreeSelectionEvent e)
		{
			updateProperties(e.getNewLeadSelectionPath());
		}
	}

	private class TreeMouseListener extends MouseAdapter
	{
		public void mouseClicked(MouseEvent e)
		{
			if ((e.isPopupTrigger() || e.getButton()==MouseEvent.BUTTON3) && e.getClickCount()>0 && e.getSource() instanceof DynamicTree)
			{
				DynamicTree tree=(DynamicTree)e.getSource();

				List inserts=new ArrayList();
				List selects=new ArrayList();

				TreePath[] selectionPaths=tree.getSelectionPaths();
				if (selectionPaths!=null && selectionPaths.length>0)
				{
					List tables=new ArrayList();
					ListMap columnTables=new ListMap();
					boolean prependSchema=false;

					for (int i=0; i<selectionPaths.length; i++)
					{
						TreePath treePath=selectionPaths[i];
						Object node=treePath.getLastPathComponent();
						if (node instanceof DynamicTreeNode)
						{
							DynamicTreeNode treeNode=(DynamicTreeNode)node;
							Object userObject=treeNode.getUserObject();
							if (userObject instanceof DatabaseTable)
							{
								if (treeNode.getParent(2) instanceof SchemaNode) prependSchema=true;
								tables.add(userObject);
							}
							else if (userObject instanceof DatabaseColumn)
							{
								if (treeNode.getParent(4) instanceof SchemaNode) prependSchema=true;
								DatabaseColumn column=(DatabaseColumn)userObject;
								columnTables.add(column.getTable(), column);
							}
						}
					}

					if (tables.size()==1 && columnTables.size()==0)
					{
						DatabaseTable table=(DatabaseTable)tables.get(0);
						String tableName=table.getTableName();
						if (prependSchema) tableName=table.getSchema().getSchemaName()+"."+tableName;

						inserts.add(new InsertTextAction(tableName));
						inserts.add("-");
						inserts.add(new InsertTextAction("select * from "+tableName));
						selects.add(new ExecuteAction("select * from "+tableName, true));
						inserts.add(new InsertTextAction("delete from "+tableName));
						selects.add(new ExecuteAction("delete from "+tableName, false));
					}
					else if (tables.size()==0 && columnTables.size()==1)
					{
						DatabaseTable table=(DatabaseTable)columnTables.keySet().iterator().next();
						String tableName=table.getTableName();
						if (prependSchema) tableName=table.getSchema().getSchemaName()+"."+tableName;

						List columns=columnTables.get(table);
						String columnNames=StringUtils.enumerate(columns, ", ");

						inserts.add(new InsertTextAction(columnNames));
						inserts.add("-");
						inserts.add(new InsertTextAction("select "+columnNames+" from "+tableName));
						selects.add(new ExecuteAction("select "+columnNames+" from "+tableName, true));
						if (columns.size()==1)
						{
							inserts.add(new InsertTextAction("select * from "+tableName+" where "+columnNames+"="));
							selects.add(new ExecuteAction("select * from "+tableName+" where "+columnNames+"=", false));
						}
					}
				}
				e.consume();
				JPopupMenu popup=null;
				if (!inserts.isEmpty())
				{
					JMenu menuInsert=new JMenu("Insert");
					for (Iterator it=inserts.iterator(); it.hasNext();)
					{
						Object item=it.next();
						if (item instanceof Action)
							menuInsert.add((Action)item);
						else
							menuInsert.addSeparator();
					}

					popup=new JPopupMenu();
					popup.add(menuInsert);
				}
				if (!selects.isEmpty())
				{
					JMenu menuSelect=new JMenu("Execute");
					for (Iterator it=selects.iterator(); it.hasNext();)
					{
						Object item=it.next();
						if (item instanceof Action)
							menuSelect.add((Action)item);
						else
							menuSelect.addSeparator();
					}

					if (popup==null) popup=new JPopupMenu();
					popup.add(menuSelect);
				}
				if (popup!=null) popup.show(tree, e.getX(), e.getY());
			}
		}
	}

	private class InsertTextAction extends AbstractAction
	{
		private String text;

		public InsertTextAction(String text)
		{
			super(text);
			this.text=text;
		}

		public void actionPerformed(ActionEvent e)
		{
			SQLPluginPanel mainPanel=SQLPluginPanel.getInstance(project, false);
			if (mainPanel!=null)
			{
				QueryPanel queryPanel=mainPanel.getActiveQueryPanel();
				if (queryPanel!=null)
				{
					if (project!=null)
					{
						ToolWindowManager manager=ToolWindowManager.getInstance(project);
						ToolWindow console=manager.getToolWindow(SQLQueryPlugin.SQL_TOOL_WINDOW);
						if (console!=null)
						{
							console.show(null);
							console.activate(null);
						}
					}
					else
						mainPanel.getTopLevelAncestor().setVisible(true);
					queryPanel.insertString(text);
				}
			}
		}
	}

	private class ExecuteAction extends AbstractAction
	{
		private String text;
		private boolean execute;

		public ExecuteAction(String text, boolean execute)
		{
			super(text);
			this.text=text;
			this.execute=execute;
		}

		public void actionPerformed(ActionEvent e)
		{
			SQLPluginPanel mainPanel=SQLPluginPanel.getInstance(project, false);
			if (mainPanel!=null)
			{
				if (project!=null)
				{
					ToolWindowManager manager=ToolWindowManager.getInstance(project);
					ToolWindow console=manager.getToolWindow(SQLQueryPlugin.SQL_TOOL_WINDOW);
					if (console!=null)
					{
						console.show(null);
						console.activate(null);
					}
				}
				else
					mainPanel.getTopLevelAncestor().setVisible(true);
				mainPanel.createQuery(text, execute);
			}
		}
	}

	private class CurrentDatabaseListener implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			if ("database".equals(e.getActionCommand()))
			{
				Object selectedItem=cbxDatabases.getSelectedItem();
				if (selectedItem instanceof Database)
					DatabaseManager.getInstance(project).setCurrentDatabase((Database)selectedItem);
				else
					DatabaseManager.getInstance(project).setCurrentDatabase(null);
			}
			else if ("group".equals(e.getActionCommand()))
			{
				Object selectedItem=cbxGroups.getSelectedItem();
				if (selectedItem instanceof String)
					DatabaseManager.getInstance(project).setCurrentGroup((String)selectedItem);
				else
					DatabaseManager.getInstance(project).setCurrentGroup(null);
			}
		}
	}

	private DynamicTree getTree(Database database)
	{
		DynamicTree tree=(DynamicTree)treeMap.get(database);
		if (tree==null)
		{
			tree=new DynamicTree();
//			tree.setBackground(Color.white);
			tree.setCellRenderer(new DynamicTreeNodeRenderer());
			DynamicTreeNode rootNode;
			if (database!=null)
				rootNode=new DatabaseNode(project, database, null);
			else
				rootNode=new InfoNode("No database selected", InfoNode.INFO, null);
			DefaultTreeModel treeModel=new DefaultTreeModel(rootNode);
			rootNode.setTreeModel(treeModel);
			tree.setModel(treeModel);
			treeMap.put(database, tree);
		}
		return tree;
	}

	private void updateTree()
	{
		Database current=DatabaseManager.getInstance(project).getCurrentDatabase();
		setTree(getTree(current));
	}

	private class DatabaseManagerObserver implements Observer
	{
		public void update(Observable o, Object arg)
		{
			NotifyObject note=(NotifyObject)arg;
			DatabaseManager databaseManager=DatabaseManager.getInstance(project);
			if ("databases changed".equals(note.getArgument(0)))
			{
				List groups=new LinkedList(databaseManager.getGroups());
				Collections.sort(groups);
				groups.add(0, DatabaseManager.ALL_GROUPS);
				cbxGroups.setModel(new DefaultComboBoxModel(groups.toArray()));

				String currentGroup=databaseManager.getCurrentGroup();
				Database currentDatabase=databaseManager.getCurrentDatabase();
				List databases=new LinkedList(databaseManager.getDatabases(currentGroup));
				databases.add(DatabaseManager.NO_DATABASE);
				cbxDatabases.setModel(new DefaultComboBoxModel(databases.toArray()));

				if (currentGroup!=null)
					cbxGroups.setSelectedItem(currentGroup);
				else
					cbxGroups.setSelectedItem(DatabaseManager.ALL_GROUPS);
				if (currentDatabase!=null)
				{
					cbxDatabases.setSelectedItem(currentDatabase);
					cbxDatabases.setToolTipText(currentDatabase.getURL());
				}
				else
				{
					cbxDatabases.setSelectedItem(DatabaseManager.NO_DATABASE);
					cbxDatabases.setToolTipText(null);
				}
			}
			else if ("current database changed".equals(note.getArgument(0)))
			{
				Object selected=cbxDatabases.getSelectedItem();
				Object currentDatabase=databaseManager.getCurrentDatabase();
				if (currentDatabase==null) currentDatabase=DatabaseManager.NO_DATABASE;
				if (selected!=currentDatabase) cbxDatabases.setSelectedItem(currentDatabase);
				if (currentDatabase instanceof Database)
					cbxDatabases.setToolTipText(((Database)currentDatabase).getURL());
				else
					cbxDatabases.setToolTipText(null);
				updateTree();
			}
			else if ("current group changed".equals(note.getArgument(0)))
			{
				Object selected=cbxGroups.getSelectedItem();
				Object currentGroup=databaseManager.getCurrentGroup();
				if (currentGroup==null) currentGroup=DatabaseManager.NO_DATABASE;
				if (selected!=currentGroup) cbxGroups.setSelectedItem(currentGroup);

				Database currentDatabase=databaseManager.getCurrentDatabase();
				List databases=new LinkedList(databaseManager.getDatabases(databaseManager.getCurrentGroup()));
				databases.add(DatabaseManager.NO_DATABASE);
				cbxDatabases.setModel(new DefaultComboBoxModel(databases.toArray()));
				if (currentDatabase!=null)
				{
					cbxDatabases.setSelectedItem(currentDatabase);
					cbxDatabases.setToolTipText(currentDatabase.getURL());
				}
				else
				{
					cbxDatabases.setSelectedItem(DatabaseManager.NO_DATABASE);
					cbxDatabases.setToolTipText(null);
				}
			}
		}
	}

	private class RefreshAction extends AbstractAction
	{
		public RefreshAction()
		{
			super("Refresh", IconManager.getIcon(AboutAction.class.getClassLoader(), IconNames.REFRESH));
		}

		public void actionPerformed(ActionEvent e)
		{
			if (cbxDatabases.getSelectedItem() instanceof Database)
			{
				Database current=(Database)cbxDatabases.getSelectedItem();
				DynamicTree tree=(DynamicTree)treeMap.get(current);
				if (tree!=null)
				{
					tree.removeTreeSelectionListener(selectionListener);
					tree.removeMouseListener(mouseListener);
				}
				treeMap.remove(current);
				setTree(getTree(current));
			}
		}
	}

	private class CancelAction extends AbstractAction
	{
		public CancelAction()
		{
			super("Close browser", IconManager.getIcon(QueryPanel.class.getClassLoader(), IconNames.CANCEL));
			putValue(Action.SHORT_DESCRIPTION, "Close this tool window.");
		}

		public void actionPerformed(ActionEvent e)
		{
			closeInstance(project);
			ToolWindowManager toolWindowManager=ToolWindowManager.getInstance(project);
			toolWindowManager.unregisterToolWindow("SQL Schema");
		}
	}
}
