/*
 * Copyright (C) 2002-2005 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.db.sql;

import java.util.List;

import com.kiwisoft.db.StatementList;
import com.kiwisoft.utils.Utils;
import junit.framework.TestCase;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.3 $, $Date: 2005/08/10 18:54:52 $
 */
public class SQLParserTest extends TestCase
{
	public SQLParserTest()
	{
		Utils.setDialogWrapper(false);
	}

	public void testSQLParser()
	{
		String statement="SeleCT *\tFRom    uSer \nWHere id<2;";
		List statements=SQLParser.parse(statement).getStatements();
		assertEquals(1, statements.size());
		SQLStatement sqlStatement=(SQLStatement)statements.get(0);
		assertEquals(statement.substring(0, statement.length()-1), sqlStatement.getText());
		assertEquals("select * from user where id<2", sqlStatement.getNormalizedText());

		statements=SQLParser.parse("select * from user; select *  from tables").getStatements();
		assertEquals(2, statements.size());
		sqlStatement=(SQLStatement)statements.get(0);
		assertEquals("select * from user", sqlStatement.getText());
		assertEquals("select * from user", sqlStatement.getNormalizedText());
		sqlStatement=(SQLStatement)statements.get(1);
		assertEquals(" select *  from tables", sqlStatement.getText());
		assertEquals("select * from tables", sqlStatement.getNormalizedText());

		statements=SQLParser.parse("/* test */").getStatements();
		assertEquals(1, statements.size());
		sqlStatement=(SQLStatement)statements.get(0);
		assertEquals("/* test */", sqlStatement.getText());
		assertEquals("", sqlStatement.getNormalizedText());

		statements=SQLParser.parse("select * /*, test */ from test").getStatements();
		assertEquals(1, statements.size());
		sqlStatement=(SQLStatement)statements.get(0);
		assertEquals("select * /*, test */ from test", sqlStatement.getText());
		assertEquals("select * from test", sqlStatement.getNormalizedText());

		statements=SQLParser.parse("select * from test; -- comment\nselect a-b from test").getStatements();
		assertEquals(2, statements.size());
		sqlStatement=(SQLStatement)statements.get(0);
		assertEquals("select * from test", sqlStatement.getText());
		assertEquals("select * from test", sqlStatement.getNormalizedText());
		sqlStatement=(SQLStatement)statements.get(1);
		assertEquals(" -- comment\nselect a-b from test", sqlStatement.getText());
		assertEquals("select a-b from test", sqlStatement.getNormalizedText());

		statements=SQLParser.parse("select\n/*test*/").getStatements();
		assertEquals(1, statements.size());
		sqlStatement=(SQLStatement)statements.get(0);
		assertEquals("select\n/*test*/", sqlStatement.getText());
		assertEquals("select", sqlStatement.getNormalizedText());

		statements=SQLParser.parse("select\n/select").getStatements();
		assertEquals(2, statements.size());
		sqlStatement=(SQLStatement)statements.get(0);
		assertEquals("select", sqlStatement.getNormalizedText());
		sqlStatement=(SQLStatement)statements.get(1);
		assertEquals("select", sqlStatement.getNormalizedText());
	}

	public void testSuccessMessage()
	{
		SQLStatement statement=(SQLStatement)SQLParser.parse("   SELECT * from user").getStatements().get(0);
		assertEquals("2 row(s) found.", statement.getSuccessMessage(2));

		statement=(SQLStatement)SQLParser.parse("   CReate table ab").getStatements().get(0);
		assertEquals("Table created.", statement.getSuccessMessage(1));
	}

	public void testComments()
	{
		DefaultSQLStatement statement;
		StatementList statementList;

		statementList=SQLParser.parse("select col1 c1 /*test*/ from test t");
		assertEquals(1, statementList.getStatements().size());
		statement=(DefaultSQLStatement)statementList.getStatement(0);
		assertEquals("select col1 c1 from test t", statement.getNormalizedText());

		statementList=SQLParser.parse("/*test*/select col1 c1 /*test*/ from test t");
		assertEquals(1, statementList.getStatements().size());
		statement=(DefaultSQLStatement)statementList.getStatement(0);
		assertEquals("select col1 c1 from test t", statement.getNormalizedText());

		statementList=SQLParser.parse("/*test*/\nselect col1 c1 /*test*/ from test t");
		assertEquals(1, statementList.getStatements().size());
		statement=(DefaultSQLStatement)statementList.getStatement(0);
		assertEquals("select col1 c1 from test t", statement.getNormalizedText());

		statementList=SQLParser.parse("--test\nselect col1 c1 from test t");
		assertEquals(1, statementList.getStatements().size());
		statement=(DefaultSQLStatement)statementList.getStatement(0);
		assertEquals("select col1 c1 from test t", statement.getNormalizedText());

		statementList=SQLParser.parse("select col1 c1 from test t --test");
		assertEquals(1, statementList.getStatements().size());
		statement=(DefaultSQLStatement)statementList.getStatement(0);
		assertEquals("select col1 c1 from test t", statement.getNormalizedText());

		statementList=SQLParser.parse("select col1 c1 from test t /*test*/");
		assertEquals(1, statementList.getStatements().size());
		statement=(DefaultSQLStatement)statementList.getStatement(0);
		assertEquals("select col1 c1 from test t", statement.getNormalizedText());
	}

	public void testSelect()
	{
		assertSelect(SQLParser.parse("select * from tables").getStatement(0), true);
		assertSelect(SQLParser.parse("select test from tables").getStatement(0), true);
		assertSelect(SQLParser.parse("select col1 test, col2 from tables").getStatement(0), true);
		assertSelect(SQLParser.parse("select col1 'test', col2 from tables").getStatement(0), true);
		assertSelect(SQLParser.parse("select col1 \"test\", col2 from tables").getStatement(0), true);
		assertSelect(SQLParser.parse("select col1 as test, col2 from tables").getStatement(0), true);
		assertSelect(SQLParser.parse("select col1 as 'test', col2 from tables").getStatement(0), true);
		assertSelect(SQLParser.parse(" select col1, col2 from tables").getStatement(0), true);
		assertSelect(SQLParser.parse(" select col1, col2 from tables where col1=2").getStatement(0), true);
		assertSelect(SQLParser.parse(" select col1, col2 from tables order by col1").getStatement(0), true);

		SQLStatement statement=SQLParser.parse(" select col1, col2 from table1, ").getStatement(0);
		assertSelect(statement, false);
		assertDescribe(statement, false);
		statement=SQLParser.parse(" select table1.col1 from table1").getStatement(0);
		assertSelect(statement, false);
		assertDescribe(statement, false);
		statement=SQLParser.parse("create table test ()").getStatement(0);
		assertSelect(statement, false);
		assertDescribe(statement, false);
	}

	public void testSimpleSelect()
	{
		DefaultSQLStatement statement;

		statement=(DefaultSQLStatement)SQLParser.parse("select * from test").getStatement(0);
		assertTrue(statement.isSimpleSelect());
		assertEquals("test", statement.getTable().getName());

		statement=(DefaultSQLStatement)SQLParser.parse("select col1, col2 from test").getStatement(0);
		assertTrue(statement.isSimpleSelect());
		assertEquals("test", statement.getTable().getName());
		assertEquals("col1", ((NameInfo)statement.getColumnInfos().get(0)).getName());
		assertEquals("col1", ((NameInfo)statement.getColumnInfos().get(0)).getLabel());
		assertEquals("col2", ((NameInfo)statement.getColumnInfos().get(1)).getName());
		assertEquals("col2", ((NameInfo)statement.getColumnInfos().get(1)).getLabel());

		statement=(DefaultSQLStatement)SQLParser.parse("select col1 c1, col2 c2 from test t").getStatement(0);
		assertTrue(statement.isSimpleSelect());
		assertEquals("test", statement.getTable().getName());
		assertEquals("t", statement.getTable().getLabel());
		assertEquals("col1", ((NameInfo)statement.getColumnInfos().get(0)).getName());
		assertEquals("c1", ((NameInfo)statement.getColumnInfos().get(0)).getLabel());
		assertEquals("col2", ((NameInfo)statement.getColumnInfos().get(1)).getName());
		assertEquals("c2", ((NameInfo)statement.getColumnInfos().get(1)).getLabel());

		statement=(DefaultSQLStatement)SQLParser.parse("select col1 as 'c1', col2 as 'c2' from test as 't'").getStatement(0);
		assertTrue(statement.isSimpleSelect());
		assertEquals("test", statement.getTable().getName());
		assertEquals("t", statement.getTable().getLabel());
		assertEquals("col1", ((NameInfo)statement.getColumnInfos().get(0)).getName());
		assertEquals("c1", ((NameInfo)statement.getColumnInfos().get(0)).getLabel());
		assertEquals("col2", ((NameInfo)statement.getColumnInfos().get(1)).getName());
		assertEquals("c2", ((NameInfo)statement.getColumnInfos().get(1)).getLabel());
	}

	public void testDescribe()
	{
		DescribeStatement statement=(DescribeStatement)SQLParser.parse(" describe schema.table ").getStatement(0);
		assertTrue(statement.isValid());
		assertEquals("schema", statement.getSchema());
		assertEquals("table", statement.getTable().getName());

		statement=(DescribeStatement)SQLParser.parse(" describe table_1").getStatement(0);
		assertTrue(statement.isValid());
		assertEquals("table_1", statement.getTable().getName());

		statement=(DescribeStatement)SQLParser.parse(" describe table ").getStatement(0);
		assertTrue(statement.isValid());
		assertNull(statement.getSchema());
		assertEquals("table", statement.getTable().getName());

		statement=(DescribeStatement)SQLParser.parse(" describe schema+table ").getStatement(0);
		assertFalse(statement.isValid());
	}

	public void testTemplate()
	{
		StatementList statementList=SQLParser.parse("@test \"table 1\", table2; select * from table3");
		assertEquals("select * from table 1", statementList.getStatement(0).getNormalizedText());
		assertEquals("select * from table2", statementList.getStatement(1).getNormalizedText());
		assertEquals("select * from table3", statementList.getStatement(2).getNormalizedText());
	}

	private void assertDescribe(SQLStatement statement, boolean value)
	{
		if (statement instanceof DescribeStatement)
		{
			assertEquals(value, ((DescribeStatement)statement).isValid());
		}
		else
			assertFalse(value);
	}

	private void assertSelect(SQLStatement statement, boolean value)
	{
		if (statement instanceof DefaultSQLStatement)
		{
			DefaultSQLStatement defaultSQLStatement=(DefaultSQLStatement)statement;
			assertEquals(value, defaultSQLStatement.isSimpleSelect());
		}
		else
			assertFalse(value);
	}
}
