/*
 * Copyright (C) 2002-2005 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.utils;

import java.awt.*;
import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.URLDecoder;
import javax.swing.*;
import javax.swing.border.EmptyBorder;

import org.jdom.Attribute;
import org.jdom.Element;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.roots.ProjectRootManager;
import com.intellij.openapi.vfs.VirtualFile;

import com.kiwisoft.utils.idea.DefaultDialogWrapper;
import com.kiwisoft.utils.idea.DialogStub;
import com.kiwisoft.utils.idea.DialogWrapper;
import com.kiwisoft.utils.idea.IdeaDialogWrapper;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.10 $, $Date: 2005/10/29 11:18:20 $
 */
public class Utils
{
	private static boolean USE_DIALOG_WRAPPER=true;

	private Utils()
	{
	}

	public static void setDialogWrapper(boolean value)
	{
		USE_DIALOG_WRAPPER=value;
	}

	public static boolean isPluginMode()
	{
		return USE_DIALOG_WRAPPER;
	}

	public static void wrapDialog(final Project project, final DialogStub dialogStub, boolean wait, final boolean modal)
	{
		Runnable runnable=new Runnable()
		{
			public void run()
			{
				DialogWrapper dialogWrapper;
				if (USE_DIALOG_WRAPPER) dialogWrapper=new IdeaDialogWrapper(project, dialogStub, modal);
				else dialogWrapper=new DefaultDialogWrapper(dialogStub, modal);
				dialogWrapper.show();
			}
		};
		if (SwingUtilities.isEventDispatchThread()) runnable.run();
		else if (wait)
		{
			try
			{
				SwingUtilities.invokeAndWait(runnable);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
		else SwingUtilities.invokeLater(runnable);
	}

	public static void arrangeWindow(Window parent, Window child)
	{
		Point posParent;
		if (parent!=null)
			posParent=parent.getLocation();
		else
			posParent=new Point();
		Dimension sizeParent;
		if (parent!=null)
			sizeParent=parent.getSize();
		else
			sizeParent=Toolkit.getDefaultToolkit().getScreenSize();
		Dimension sizeChild=child.getSize();
		Point posChild=new Point();
		posChild.x=Math.max(0, posParent.x+(sizeParent.width-sizeChild.width)/2);
		posChild.y=Math.max(0, posParent.y+(sizeParent.height-sizeChild.height)/2);
		child.setLocation(posChild);
	}

	public static JButton createButton(Action action)
	{
		JButton button=new JButton(action);
		button.setMargin(new Insets(0, 0, 0, 0));
		button.setBorder(new EmptyBorder(3, 3, 3, 3));
		button.setText(null);
		button.setFocusable(false);
		KeyStroke keyStroke=(KeyStroke)action.getValue(Action.ACCELERATOR_KEY);
		if (keyStroke!=null)
		{
			InputMap inputMap=button.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
			inputMap.put(keyStroke, Action.ACCELERATOR_KEY);
			button.getActionMap().put(Action.ACCELERATOR_KEY, action);
		}
		return button;
	}

	public static JToggleButton createToggleButton(Action action)
	{
		JToggleButton button=new JToggleButton(action);
		button.setMargin(new Insets(1, 1, 1, 1));
		button.setText(null);
		button.setBorderPainted(false);
		KeyStroke keyStroke=(KeyStroke)action.getValue(Action.ACCELERATOR_KEY);
		if (keyStroke!=null)
		{
			InputMap inputMap=button.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
			inputMap.put(keyStroke, Action.ACCELERATOR_KEY);
			button.getActionMap().put(Action.ACCELERATOR_KEY, action);
		}
		return button;
	}

	/**
	 * Reads a boolean attribute from the specified Element
	 * @param child         the <code>Element</code> containing the attribute
	 * @param attributeName the <code>String</code> name of the attribute
	 * @param defaultValue  the <code>boolean</code> default value, if the attribute does not exists
	 * @return  the boolean value as stored as attribute value of the specified element
	 */
	public static boolean getBoolean(Element child, String attributeName, boolean defaultValue)
	{
        Attribute attribute=child.getAttribute(attributeName);
        if (attribute!=null)
        {
    		String bool=child.getAttributeValue(attributeName);
	    	if (bool!=null) return Boolean.valueOf(bool).booleanValue();
		    else return defaultValue;
        }
        else return defaultValue;
	}

	/**
	 * Reads a boolean attribute from the specified Element
	 * @param child         the <code>Element</code> containing the attribute
	 * @param attributeName the <code>String</code> name of the attribute
	 * @param defaultValue  the <code>boolean</code> default value, if the attribute does not exists
	 * @return  the boolean value as stored as attribute value of the specified element
	 */
	public static Boolean getBoolean(Element child, String attributeName, Boolean defaultValue)
	{
		Attribute attribute=child.getAttribute(attributeName);
		if (attribute!=null)
		{
			String bool=attribute.getValue();
			if (bool!=null) return Boolean.valueOf(bool);
			else return defaultValue;
		}
		else return defaultValue;
	}

	/**
	 * Reads a boolean attribute from the specified Element
	 * @param child         the <code>Element</code> containing the attribute
	 * @param attributeName the <code>String</code> name of the attribute
	 * @param defaultValue  the <code>boolean</code> default value, if the attribute does not exists
	 * @return  the boolean value as stored as attribute value of the specified element
	 */
	public static String getString(Element child, String attributeName, String defaultValue)
	{
		Attribute attribute=child.getAttribute(attributeName);
		if (attribute!=null)
		{
			String value=attribute.getValue();
			return value!=null ? value : defaultValue;
		}
		return defaultValue;
	}

	/**
	 * Reads an int attribute from the specified Element
	 * @param child         the <code>Element</code> containing the attribute
	 * @param attributeName the <code>String</code> name of the attribute
	 * @param defaultValue  the <code>int</code> default value, if the attribute does not exists
	 * @return  the boolean value as stored as attribute value of the specified element
	 */
	public static Integer getInteger(Element child, String attributeName, Integer defaultValue)
	{
		String value=child.getAttributeValue(attributeName);
		if (value!=null)
		{
			try
			{
				return Integer.valueOf(value);
			}
			catch (NumberFormatException e)
			{
				return defaultValue;
			}
		}
		else
			return defaultValue;
	}

	public static boolean setValue(Element child, String name, Object content)
	{
		if (content!=null)
		{
			child.setAttribute(name, String.valueOf(content));
			return true;
		}
		return false;
	}

	public static File getExistingPath(String exportPath)
	{
		File exportDir=null;
		if (exportPath!=null)
		{
			exportDir=new File(exportPath);
			if (!exportDir.exists() || !exportDir.isDirectory() || !exportDir.canWrite())
			{
				exportDir=null;
			}
		}
		if (exportDir==null) exportDir=new File(System.getProperty("user.home"));
		return exportDir;
	}

	public static ClassLoader getProjectClassLoader(Project project)
	{
		if (project==null) Utils.class.getClassLoader();
		//noinspection deprecation
		VirtualFile[] roots=ProjectRootManager.getInstance(project).getFullClassPath();
		URL[] classpath=new URL[roots.length];
		for (int i=0; i<roots.length; i++)
		{
			try
			{
				String path=roots[i].getPath();
				if (path.endsWith("!/")) path=path.substring(0, path.length()-2);
				classpath[i]=new File(path).toURL();
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
		return new URLClassLoader(classpath, Utils.class.getClassLoader());
	}

	public static File getPluginBasePath(Class aClass)
	{
		String className=aClass.getName();
		className=className.replace('.', '/')+".class";
		URL url=aClass.getClassLoader().getResource(className);
		if ("file".equals(url.getProtocol()))
		{
			String dir=url.getFile();
			dir=dir.substring(0, dir.length()-className.length());
			return new File(dir).getParentFile();
		}
		if ("jar".equals(url.getProtocol()))
		{
			String dir=url.getFile();
			if (dir.startsWith("file:/"))
			{
				String jarFile=dir.substring(6, dir.indexOf('!'));
				File file=new File(jarFile).getParentFile();
				return file.getParentFile();
			}
		}
		throw new RuntimeException("Couldn't locate base directory.");
	}

	public static File getClassBasePath(Class aClass)
	{
		String className=aClass.getName();
		className=className.replace('.', '/')+".class";
		URL url=aClass.getClassLoader().getResource(className);
		if ("file".equals(url.getProtocol()))
		{
			String dir=url.getFile();
			dir=dir.substring(0, dir.length()-className.length());
			try
			{
				dir=URLDecoder.decode(dir, "UTF-8");
			}
			catch (UnsupportedEncodingException e)
			{
			}
			return new File(dir);
		}
		if ("jar".equals(url.getProtocol()))
		{
			String dir=url.getFile();
			if (dir.startsWith("file:/"))
			{
				String jarFile=dir.substring(6, dir.indexOf('!'));
				try
				{
					jarFile=URLDecoder.decode(jarFile, "UTF-8");
				}
				catch (UnsupportedEncodingException e)
				{
				}
				return new File(jarFile);
			}
		}
		throw new RuntimeException("Couldn't locate base directory.");
	}

	public static boolean equals(Object o1, Object o2)
	{
		if (o1==o2) return true;
		else if (o1==null) return false; // o2 is not null because of the first test
		else return o1.equals(o2);
	}
}
