/*
 * Copyright (C) 2002-2005 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.utils.gui.table;

import java.util.*;
import javax.swing.table.TableCellRenderer;

import com.kiwisoft.utils.DoubleKeyMap;
import com.kiwisoft.utils.Tristate;
import com.kiwisoft.utils.SetMap;
import com.kiwisoft.utils.format.*;
import com.kiwisoft.utils.gui.ObjectStyle;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.3 $, $Date: 2005/08/10 18:55:29 $
 */
public class RendererFactory
{
	private static RendererFactory instance;

	public synchronized static RendererFactory getInstance()
	{
		if (instance==null) instance=new RendererFactory();
		return instance;
	}

	private Map defaultRenderers;
	private DoubleKeyMap renderers=new DoubleKeyMap();
	private DoubleKeyMap createdRenderers=new DoubleKeyMap();
	private DoubleKeyMap styleRenderers=new DoubleKeyMap();

	private RendererFactory()
	{
		defaultRenderers=new HashMap();
		defaultRenderers.put(Object.class, new DefaultObjectRenderer(new DefaultObjectFormat(ObjectFormat.DEFAULT)));
		defaultRenderers.put(Number.class, new DefaultObjectRenderer(new DefaultNumberFormat(ObjectFormat.DEFAULT)));
		defaultRenderers.put(Date.class, new DefaultObjectRenderer(new DefaultDateFormat(ObjectFormat.DEFAULT)));
		defaultRenderers.put(String.class, new DefaultObjectRenderer(new DefaultStringFormat(ObjectFormat.DEFAULT)));
		defaultRenderers.put(Boolean.class, new BooleanObjectRenderer());
		defaultRenderers.put(Tristate.class, new TristateObjectRenderer());

		renderers.put(Boolean.class, "Tristate", new TristateObjectRenderer());
	}

	public SetMap getVariants(Class objectClass)
	{
		SetMap variants=new SetMap();
		variants.add(null, ObjectFormat.DEFAULT);
		Class aClass=objectClass;
		while (aClass!=null)
		{
			for (Iterator it=renderers.getKeys(aClass).iterator(); it.hasNext();)
			{
				variants.add(null, it.next());
			}
			aClass=aClass.getSuperclass();
		}
		for (Iterator it=FormatManager.getInstance().getFormats().iterator(); it.hasNext();)
		{
			ObjectFormat format=(ObjectFormat) it.next();
			if (format.canFormat(objectClass)) variants.add(format.getGroup(), format.getName());
		}
		return variants;
	}

	public ObjectRenderer getRenderer(Class aClass, String variant)
	{
		if (variant==null || ObjectFormat.DEFAULT.equals(variant))
		{
			ObjectRenderer renderer=getDefaultRenderer(aClass);
			defaultRenderers.put(aClass, renderer);
			return renderer;
		}
		else
		{
			ObjectRenderer renderer=getVariantRenderer(aClass, variant);
			if (renderer==null) renderer=getRenderer(aClass, null);
			createdRenderers.put(aClass, variant, renderer);
			return renderer;
		}
	}

	private ObjectRenderer getVariantRenderer(Class aClass, String variant)
	{
		ObjectRenderer renderer=(ObjectRenderer) renderers.get(aClass, variant);
		if (renderer==null) renderer=(ObjectRenderer) createdRenderers.get(aClass, variant);
		if (renderer==null)
		{
			for (Iterator it=FormatManager.getInstance().getFormats().iterator(); it.hasNext();)
			{
				ObjectFormat format=(ObjectFormat) it.next();
				if (variant.equals(format.getName()) && format.canFormat(aClass))
				{
					return new DefaultObjectRenderer(format);
				}
			}
			if (aClass.getSuperclass()!=null) renderer=getVariantRenderer(aClass.getSuperclass(), variant);
		}
		return renderer;
	}

	private ObjectRenderer getDefaultRenderer(Class aClass)
	{
		if (aClass==null) return null;
		ObjectRenderer renderer=(ObjectRenderer) defaultRenderers.get(aClass);
		if (renderer==null)
		{
			for (Iterator it=FormatManager.getInstance().getFormats().iterator(); it.hasNext();)
			{
				ObjectFormat format=(ObjectFormat) it.next();
				if ((format.getName()==null || ObjectFormat.DEFAULT.equals(format.getName())) && format.canFormat(aClass))
				{
					renderer=new DefaultObjectRenderer(format);
					break;
				}
			}
			if (renderer==null)
			{
				Class[] interfaces=aClass.getInterfaces();
				for (int i=0; i<interfaces.length && renderer==null; i++)
				{
					Class anInterface=interfaces[i];
                    renderer=getDefaultRenderer(anInterface);
				}
			}
			if (renderer==null) renderer=getDefaultRenderer(aClass.getSuperclass());
		}
		return renderer;
	}

	public TableCellRenderer getRenderer(Class aClass, String variant, ObjectStyle style)
	{
		ObjectRenderer renderer=getRenderer(aClass, variant);
		if (style==null) return renderer;
		StyleRenderer styleRenderer=(StyleRenderer) styleRenderers.get(renderer, style);
		if (styleRenderer==null)
		{
			styleRenderer=new StyleRenderer(renderer, style);
			styleRenderers.put(renderer, style, styleRenderer);
		}
		return styleRenderer;
	}

	public void setRenderer(Class aClass, ObjectFormat format)
	{
		setRenderer(aClass, format.getName(), new DefaultObjectRenderer(format));
	}

	public void setRenderer(Class aClass, String variant, ObjectRenderer renderer)
	{
		if (variant==null || ObjectFormat.DEFAULT.equals(variant))
		{
			defaultRenderers.put(aClass, renderer);
		}
		else
		{
			renderers.put(aClass, variant, renderer);
			createdRenderers.remove(aClass, variant);
		}
	}

}
